/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.geronimo.samples.dbtester.beans;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.sql.DataSource;

import org.apache.geronimo.gbean.AbstractName;
import org.apache.geronimo.gbean.AbstractNameQuery;
import org.apache.geronimo.kernel.Kernel;
import org.apache.geronimo.kernel.KernelRegistry;
import org.apache.geronimo.naming.ResourceSource;

public class DBManagerBean {

    private final Map<String, DataSource> poolMap = new HashMap<String, DataSource>();

    public DBManagerBean() {
        Kernel kernel = KernelRegistry.getSingleKernel();
        Set<AbstractName> cfList = kernel.listGBeans(new AbstractNameQuery(ResourceSource.class.getName()));

        for (AbstractName name : cfList) {

            try {
                Object rs = kernel.invoke(name, "$getResource", new Object[]{}, new String[]{});

                if (rs instanceof DataSource) {
                    DataSource ds = (DataSource) rs;
                    poolMap.put(name.getArtifact().getArtifactId(), ds);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public Map getDeployedPools() {
        return poolMap;
    }

    public Map getTableList(String poolName) throws SQLException {

        Map<String, List<String>> tableMap = new HashMap<String, List<String>>();

        if (poolMap.containsKey(poolName)) {
            DataSource ds = poolMap.get(poolName);
            Connection con = null;
            try {

                con = ds.getConnection();

                DatabaseMetaData metaData = con.getMetaData();
                String[] tableTypes = {"TABLE"};
                ResultSet rs = metaData.getTables(null, null, null, tableTypes);

                while (rs.next()) {
                    String schemaName = rs.getString("TABLE_SCHEM");
                    String tableName = rs.getString("TABLE_NAME");
                    List<String> tableList;

                    if (tableMap.containsKey(schemaName)) {
                        tableList = tableMap.get(schemaName);
                        tableList.add(tableName);
                    } else {
                        tableList = new ArrayList<String>();
                        tableList.add(tableName);
                    }
                    tableMap.put(schemaName, tableList);
                }
            } finally {
                if (con != null) {
                    try {
                        con.close();
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        return tableMap;
    }

    public List<List<String>> getTableContents(String poolName, String schemaName, String tableName) throws SQLException {

        List<List<String>> matrix = new ArrayList<List<String>>();

        DataSource ds = poolMap.get(poolName);
        if (ds == null) {
            throw new IllegalArgumentException("no pool named: " + poolName);
        }
        Connection con = ds.getConnection();

        try {
            String SQL = "SELECT * FROM ";

            if (schemaName == null || schemaName.equals("null")) {
                SQL += tableName; //Problem comes with MySQL
            } else {
                SQL += (schemaName + "." + tableName);
            }

            PreparedStatement pStmt = con.prepareStatement(SQL);
            try {
                ResultSet rs = pStmt.executeQuery();

                try {
                    ResultSetMetaData metaData = rs.getMetaData();
                    int columns = metaData.getColumnCount();

                    ArrayList<String> header = new ArrayList<String>();
                    for (int i = 1; i <= columns; i++) {
                        String colName = metaData.getColumnLabel(i);
                        header.add(colName);
                    }
                    matrix.add(header);

                    while (rs.next()) {
                        ArrayList<String> row = new ArrayList<String>();

                        for (int i = 1; i <= columns; i++) {
                            row.add(rs.getString(i));
                        }
                        matrix.add(row);
                    }
                } finally {
                    rs.close();
                }
            } finally {
                pStmt.close();
            }
        } finally {
            con.close();
        }

        return matrix;
    }

}
